/* ------------------------------ split.c ----------------------------- */
/* This program unloads InFile into floppy sized files			*/
/*									*/
/* Author:	Eyal Lebedinsky						*/
/* Date:	8 May 1992						*/
/* Version:	1.5							*/
/*									*/
/* This program is released into the public domain. Use it at		*/
/* your own risk.							*/
/*									*/
/* Change Log:								*/
/*   25 Feb 1991							*/
/*	remove -f option						*/
/*	add -s option							*/
/*   29 Sep 1991							*/
/*	added -ss option						*/
/*	does not round down to 1k size					*/
/*   8 May 1992 							*/
/*	changed output buffering to 16K					*/
/*									*/
/* todo:								*/
/*	check write() return code					*/
/*									*/
/* --------------------------------------------------------------------	*/


#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <dos.h>
#include <string.h>

#define	SEC_SIZE	512

static char inbuf[(1024*16)], outbuf[(1024*16)];
char out_name[50];   /* output file */
static char rec[SEC_SIZE], command[50];

static void
usage ()
{
   fprintf (stderr, "split v1.5 May/92 by Eyal Lebedinsky\n");
   fprintf(stderr, "usage: split <filename> [<parts-name> -v -b -e -s]\n");
   fprintf(stderr, "\t<file-name>  name of (input) full file\n");
   fprintf(stderr, "\t<parts-name> name of (output) parts on floppy\n");
   fprintf(stderr, "\t-v           verbose\n");
   fprintf(stderr, "\t-b           use drive B:\n");
   fprintf(stderr, "\t-e           erase each floppy\n");
   fprintf(stderr, "\t-snnn        maximum output part size is nnnKbyte\n");
   fprintf(stderr, "\t-ssnnn       maximum output part size is nnnSectors\n");
   exit(4);
}

void
main (argc, argv)
int argc;
char *argv[];
{
   FILE *fin, *fout;
   long i = 0, size, disk_size, tot_size = 0, psize = 0, free_size = 0;
   int verbose, erase, drive, eoflag = 0, rec_size, j, errors = 0;
   char *disk_no, *drive_name, *p, *q;
   struct diskfree_t diskspace;

   _fmode = O_BINARY;
   rec_size = 0;

   if (argc < 2)
      usage();

   if (argv[1][0] == '-') {
      fprintf (stderr, "Bad input file name %s\n", argv[1]);
      usage ();
   }

   drive_name = "A:";
   strcpy (out_name, drive_name);
   if (argc < 3 || argv[2][0] == '-') {
      for (p = argv[1] + strlen (argv[1]) - 1;	/* find start of name */
	   p >= argv[1] && *p != '\\' && *p != ':';)
	 --p;
      ++p;
      for (q = out_name + strlen (out_name);	/* find end of name*/
	   *p != '\0' && *p != '.';)
         *q++ = *p++;
      *q = '\0';
      j = 2;
   } else {
      strcat (out_name, argv[2]);
      j = 3;
   }

   verbose = erase = 0;
   psize = 0;
   drive = 1;
   
   for (; j < argc; ++j) {
      if (argv[j][0] != '-') {
         fprintf (stderr, "unknown option %s\n", argv[j]);
	 ++errors;
         continue;
      }
      switch (toupper (argv[j][1])) {
      case 'B':
	drive = 2;
	drive_name = "B:";
	break;

      case 'E':
	erase = 1;
	break;

      case 'V':
	verbose = 1;
	break;

      case 'S':
	if (toupper (argv[j][2]) == 'S') {
		if (sscanf (&argv[j][3], "%ld", &i) != 1 || i < 1) {
			fprintf (stderr, "invalid part size %s\n",
					argv[j]);
			++errors;
			break;
		}
		psize = i * SEC_SIZE * 1L;
		break;
	}
	if (sscanf (&argv[j][2], "%ld", &i) != 1 || i < 1) {
		fprintf (stderr, "invalid part size %s\n",
				argv[j]);
		++errors;
		break;
	}
	psize = i * 1024L;
	break;

      default:
	fprintf (stderr, "unknown option %s\n", argv[j]);
	++errors;
	break;
      }
   }

   if (errors)
   	usage ();

   out_name[0] = drive_name[0];
   
   if ((fin = fopen (argv[1], "rb")) == NULL) {
      printf ("\nCan not open %s\n", argv[1]);
      exit (4);
   }
   if (setvbuf (fin, inbuf, _IOFBF, (1024*16))) {
	printf ("input buffering failed\n");
	exit (4);
   }

   disk_no = out_name + strlen (out_name) + 1;
   strcat (out_name, ".000");
   
   i = 0;
   while (!eoflag) {
      if (psize && free_size) {
	 size = free_size;
	 goto cont;
      }

      printf ("\a\nPut diskette %s in drive %s", out_name, drive_name);
      printf ("\nPress Return when ready...");
      if (getchar() == '*') {
	 printf ("split aborted, incomplete disk set.\n");
	 break;
      }

      j = 1;
again:
      _dos_getdiskfree (drive, &diskspace);
      disk_size = (long)diskspace.total_clusters *
		  (long)diskspace.sectors_per_cluster *
		  (long)diskspace.bytes_per_sector;
      free_size = (long)diskspace.avail_clusters *
		  (long)diskspace.sectors_per_cluster *
		  (long)diskspace.bytes_per_sector;

      if (free_size < disk_size && erase && j) {
      	 strcpy (command, "echo y|del ");
      	 strcat (command, drive_name);
      	 strcat (command, "\\*.*");
         system (command);
         j = 0;
         goto again;
      }

      printf ("available space on drive %s is %lu\n",
		drive_name, free_size);

      if (free_size == 0) {
	 printf
	    ("\nPlease use a disk with available space.");
	 continue;
      }
cont:
      if (psize && psize < free_size)
	 size = psize;
      else
	 size = free_size;
      free_size -= size;

      if ((fout = fopen (out_name, "wb")) == NULL) {
	 printf ("\nError opening output file\n");
	 continue;
      }
      if (setvbuf (fout, outbuf, _IOFBF, (1024*16))) {
	 printf ("output buffering failed\n");
	 exit (4);
      }

      if (disk_no[2] == '9') {
      	 disk_no[2] = '0';
	 if (disk_no[1] == '9') {
	    disk_no[1] = '0';
	    if (disk_no[0] == '9') {
	       fprintf (stderr, "Too many disks used. Aborting\n");
	       exit (4);
	    } else
      	       ++disk_no[0];
	 } else
      	    ++disk_no[1];
      } else
      	++disk_no[2];

      /* read records from input files until end of file or output file
	 is full */

      for (;;) {
	 if (rec_size == 0 &&
	     (eoflag = ((rec_size = fread (rec, 1, SEC_SIZE, fin)) == 0)))
	    break;

	 if ((size -= rec_size) < 0)   /* any more room on floppy? */
	    break;		       /* if not - quit 	   */

	 fwrite (rec, rec_size, 1, fout);
	 tot_size += rec_size;
	 rec_size = 0;

	 if (verbose)
	    printf("split %luk bytes\r", ++i);
      }
      fclose (fout);
   }

   /* write end-of-set file */
   
   if ((fout = fopen (out_name, "wb")) == NULL)
      printf ("\nError opening output file\n");
   else
      fclose (fout);

   fclose (fin);

   printf("split: %lu bytes read from %s.\n", tot_size, argv[1]);

   exit(0);
}

